class QuackamoleEventManager {
    static syntheticEvents = new Map();

    on = (eventType, callback, repeat = true) => {
        const eventListeners = this.__getEventListenersFor(eventType);
        eventListeners.set(callback, repeat);
    };

    off = (eventType, callback) => {
        const eventListeners = this.__getEventListenersFor(eventType);
        eventListeners.delete(callback);
    };

    emit = (eventType, payload, metadata) => {
        const eventListeners = this.__getEventListenersFor(eventType);

        for (const [callback, repeat] of eventListeners) {
            if (!repeat) {
                eventListeners.delete(callback);
            }
            callback(payload, metadata);
        }
    };

    __getEventListenersFor = (eventType) => {
        if (!QuackamoleEventManager.syntheticEvents.get(eventType)) {
            QuackamoleEventManager.syntheticEvents.set(eventType, new Map());
        }
        return QuackamoleEventManager.syntheticEvents.get(eventType);
    };
}

const quackamoleEventManagerSingleton = new QuackamoleEventManager();


class Quackamole {
    constructor() {
        this.eventManager = quackamoleEventManagerSingleton;
        this.__init();
    }

    broadcastData = (eventType, data) => {
        const action = {type: 'PLUGIN_SEND_TO_ALL_PEERS', payload: {eventType, data}};
        window.top.postMessage(action, '*');
    };

    sendDataTo = (peerIds, eventType, data) => {
        const action = {type: 'PLUGIN_SEND_TO_PEER', payload: {peerIds, eventType, data}};
        window.top.postMessage(action, '*');
    };

    getPeerIds = () => {
        const action = {type: 'PLUGIN_REQUEST_PEER_IDS'};
        window.top.postMessage(action, '*');
    };

    __init = () => {
        window.addEventListener('message', this.__receiveMessage);
    }

    __receiveMessage = event => {
        switch(event.data.type) {
            case 'PLUGIN_DATA': {
                // emit the event received from another peer.
                const {eventType, data} = event.data.payload;
                this.eventManager.emit(eventType, data);
            }
        }
    }
}

try {
    module.exports = {
        quackamoleEventManager: quackamoleEventManagerSingleton,
        Quackamole: Quackamole
    };
} catch(e) {}
